﻿using System;
using System.Collections.Generic;
using System.Data.SQLite;
using System.Linq;

namespace TestProject
{
    /// <summary>
    /// 표준 편차 함수
    /// </summary>
    [SQLiteFunction(Arguments = 1, FuncType = FunctionType.Aggregate, Name = "STDEV")]
    public class STDEV : SQLiteFunction
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 소스 리스트
        /// </summary>
        private List<double> sourceList = new List<double>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 단계별 처리하기 - Step(argumentArray, stepNumber, contextData)

        /// <summary>
        /// 단계별 처리하기
        /// </summary>
        /// <param name="argumentArray">인자 배열</param>
        /// <param name="stepNumber">단계 변호</param>
        /// <param name="contextData">컨텍스트 데이터</param>
        public override void Step(object[] argumentArray, int stepNumber, ref object contextData)
        {
            if(contextData == null)
            {
                contextData = 0;

                this.sourceList.Clear();
            }

            this.sourceList.Add((double)argumentArray[0]);
        }

        #endregion
        #region 최종 처리하기 - Final(contextData)

        /// <summary>
        /// 최종 처리하기
        /// </summary>
        /// <param name="contextData">컨텍스트 데이터</param>
        /// <returns>결과</returns>
        public override object Final(object contextData)
        {
            if(this.sourceList.Count == 1)
            {
                return double.NaN;
            }

            double average = this.sourceList.Average();

            double minusSquareSummary = 0.0;

            foreach(double source in sourceList)
            {
                minusSquareSummary += (source - average) * (source - average);
            }

            double stdev = Math.Sqrt(minusSquareSummary / (sourceList.Count - 1));

            return stdev;
        }

        #endregion
    }
}